function [P_sc_avg, P_sc_dev, F_Bl_avg, F_Bl_dev, rho_avg]=get_Bell_P_F(P4, N_smp, pp, n, nM, nT)
%P_sc_avg: average of success rate to emit a right photon, 
% P_sc_dev: deviation, 
% F_Bl_avg: average of fidelity of the final Bell state, 
% F_Bl_dev: deviation, 
% rho_avg: average of the density matrix entries. format: 3-by-9 matrix:
%   [[ rr, rw, ro, wr, ww, wo, or, ow oo]_{early, late};
%   [ rr, rw, ro, wr, ww, wo, or, ow oo]_{early, late};
%   sqrt(rr*rr), 0, 0, 0, 0, 0, 0, 0, 0]
% meaning: r: right photon, w: wrong photon, o: photonic vacancy. 
% rw: right photon on left and wrong photon on right.
% These 19 entries can build up a 36-by-36 density matrix with the bases:
% (left[early, late]tensor[right, wrong, empty])tensor(right[early, late]tensor[right, wrong, empty]).
% the 36-by-36 matrix can be reduced into 4-by-4 density matirx with the
% basis: [left, right]tensor[early, late] 

P_sc=zeros(N_smp,N_smp,N_smp,N_smp);
F_Bl=zeros(N_smp,N_smp,N_smp,N_smp);

P_sc_avg=0;
F_Bl_avg=0;
rho_avg=zeros(3,9);
% tic
for iA=1:N_smp
    for jA=1:N_smp
        for iB=1:iA
            if iB == iA  % vectorize it to understand
                lm=jA-1;
            else 
                lm=N_smp;
            end
            for jB=1:lm
                poss=pp(iA)*pp(jA)*pp(iB)*pp(jB);
                P_sucT=0;
                rho_avgT=zeros(3,9);
                parfor i=1:n+nM
                    for j=n+nM+(1:(n+nT))
                        
                        P_sucABEL=sum(P4([1,3,4],i, iA, jA))*sum(P4([1,3,4],j,iB, jB));
                        P_sucABLE=sum(P4([1,3,4],j,iA, jA))*sum(P4([1,3,4],i, iB, jB));
                        if P_sucABEL*P_sucABLE > 1e-12
                            P_sucT=P_sucT+(P_sucABEL+P_sucABLE);
    
                            %---------------------------------%---------------------------------
                            
                            pAE=P4([1,3,4],i, iA, jA)';
                            pAL=P4([1,3,4],j, iA, jA)';
                            pBE=P4([1,3,4],i, iB, jB)';
                            pBL=P4([1,3,4],j, iB, jB)';
                            pAE=pAE/sum(pAE);
                            pAL=pAL/sum(pAL);
                            pBE=pBE/sum(pBE);
                            pBL=pBL/sum(pBL);
    
                            pABEL=kron(pAE,pBL);
                            pABEL=P_sucABEL/(P_sucABEL+P_sucABLE)*pABEL;
                            pABLE=kron(pAL,pBE);
                            pABLE=P_sucABLE/(P_sucABEL+P_sucABLE)*pABLE;
    
                            rho=[pABEL;
                                pABLE;
                                [sqrt(pABEL(1)*pABLE(1)),zeros(1,8)]];
                            rho=rho/(sum(pABEL)+sum(pABLE));
                            rho_avgT=rho_avgT+(P_sucABEL+P_sucABLE)*rho;
    
                            %---------------------------------
                            pBAEL=kron(pBE,pAL);
                            pBAEL=P_sucABLE/(P_sucABEL+P_sucABLE)*pBAEL;
                            pBALE=kron(pBL,pAE);
                            pBALE=P_sucABEL/(P_sucABEL+P_sucABLE)*pBALE;
    
                            rho=[pBAEL;
                                pBALE;
                                [sqrt(pBAEL(1)*pBALE(1)),zeros(1,8)]];
                            rho=rho/(sum(pBAEL)+sum(pBALE));
                            rho_avgT=rho_avgT+(P_sucABEL+P_sucABLE)*rho;
                        end
                    end
                end
                P_sc_avg=P_sc_avg+2*poss*P_sucT;
                F_Bl_avg=F_Bl_avg+poss*(sum(rho_avgT(:,1))+rho_avgT(3,1))/2/P_sucT;
                rho_avg=rho_avg+poss*rho_avgT/P_sucT;

                P_sc(iA, jA, iB, jB)=P_sucT;
                P_sc(iB, jB, iA, jA)=P_sc(iA, jA, iB, jB);
                F_Bl(iA, jA, iB, jB)=(sum(rho_avgT(1:3,1))+rho_avgT(3,1))/4/P_sucT;
                F_Bl(iB, jB, iA, jA)=F_Bl(iA, jA, iB, jB);
            end
        end
        %---------------------------------%---------------------------------%---------------------------------%---------------------------------
        poss=(pp(iA)*pp(jA))^2;
        P_sucT=0;
        rho_avgT=zeros(3,9);
        parfor i=1:n+nM
            for j=n+nM+(1:(n+nT))
                
                P_sucAAEL=sum(P4([1,3,4],i,iA, jA))*sum(P4([1,3,4],j,iA, jA));
                if P_sucAAEL > 1e-12
                    P_sucT=P_sucT+2*(P_sucAAEL);
    
                    %---------------------------------%---------------------------------
                    
                    pAE=P4([1,3,4],i,iA, jA)';
                    pAL=P4([1,3,4],j,iA, jA)';
                    pAE=pAE/sum(pAE);
                    pAL=pAL/sum(pAL);
    
                    pAAEL=kron(pAE,pAL);
                    pAAEL=pAAEL/2;
                    pAALE=kron(pAL,pAE);
                    pAALE=pAALE/2;
    
                    rho=[pAAEL;
                        pAALE;
                        [sqrt(pAAEL(1)*pAALE(1)),zeros(1,8)]];
                    rho=rho/(sum(pAAEL)+sum(pAALE));
                    rho_avgT=rho_avgT+2*(P_sucAAEL)*rho;
                end
            end
        end
        P_sc_avg=P_sc_avg+poss*P_sucT;
        F_Bl_avg=F_Bl_avg+poss*(sum(rho_avgT(1:3,1))+rho_avgT(3,1))/2/P_sucT;
        rho_avg=rho_avg+poss*rho_avgT/P_sucT;

        P_sc(iA, jA, iA, jA)=P_sucT;
        F_Bl(iA, jA, iA, jA)=(sum(rho_avgT(1:3,1))+rho_avgT(3,1))/2/P_sucT;
    end
end

% toc

possi=pp'*pp;
possii=zeros(N_smp,N_smp,N_smp,N_smp);
for i=1:N_smp
    for j=1:N_smp
        possii(:,:,i,j)=pp(i)*pp(j)*possi;
    end
end

% P_sc_avg
% P_sc_avg_ck=sum(possii.*P_sc,"all")
P_sc_dev=sqrt(sum(possii.*(P_sc-P_sc_avg).^2, 'all'));

% F_Bl_avg
% F_Bl_avg_ck=sum(possii.*F_Bl,"all")
F_Bl_dev=sqrt(sum(possii.*(F_Bl-F_Bl_avg).^2, 'all'));



end